AGENT_DISCOVERY_PATH = "com.instana.plugin.python.discovery"
AGENT_TRACES_PATH = "com.instana.plugin.python/traces.%d"
AGENT_DATA_PATH = "com.instana.plugin.python.%d"
AGENT_RESPONSE_PATH = "com.instana.plugin.python/response.%d?messageId=%s"
AGENT_DEFAULT_HOST = "localhost"
AGENT_DEFAULT_PORT = 42699
AGENT_HEADER = "Instana Agent"
